#========================================================================================
# package      DE_EPAGES::DocdataPay::t::unit::CreateRequest
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for SOAP Create request.
#========================================================================================
package DE_EPAGES::DocdataPay::t::unit::CreateRequest;

use base DE_EPAGES::Core::API::Test;
use strict;

use Data::Dumper;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::DocdataPay::API::SOAP::Request::CreateRequest;
use DE_EPAGES::Test::Mock::EpagesObject;
use Readonly;
use Test::MockObject::Extends;
use Test::More;

sub test {
    my $self = shift;
    $self->_test_create_request_without_shipping_address_positive();
    $self->_test_create_request_with_shipping_address_positive();
    done_testing();
}

sub _test_create_request_without_shipping_address_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $Basket              = $self->_getMockedBasket(0);
    my $MockedCreateRequest = $self->_getMockedCreateRequest();
    my $GeneratedMessage    = $MockedCreateRequest->buildCreateRequest($Basket);
    my $ExpectedMessage     = $self->_build_create_request_message(0);
    is_deeply( $GeneratedMessage, $ExpectedMessage,
        "$SubTestName - Create message without specifying any shipping address." )
      or diag Dumper($GeneratedMessage);
    return;
}

sub _test_create_request_with_shipping_address_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $Basket              = $self->_getMockedBasket(1);
    my $MockedCreateRequest = $self->_getMockedCreateRequest();
    my $GeneratedMessage    = $MockedCreateRequest->buildCreateRequest($Basket);
    my $ExpectedMessage     = $self->_build_create_request_message(1);
    is_deeply( $GeneratedMessage, $ExpectedMessage, "$SubTestName - Create message specifying a shipping address." )
      or diag Dumper($GeneratedMessage);
    return;
}

sub _getMockedCreateRequest {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::DocdataPay::API::SOAP::Request::CreateRequest');

    $Mockify->addMockWithReturnValue( '_GetCodeByLanguageID', 'en' );

    $Mockify->addMock( '_TestObject', sub {
        shift;
        return;
    });

    $Mockify->addMock('_convertNumberToDocdataFormat', sub {
        shift;
        my ($Number) = @_;
        my $IndexDotSeparator = index( $Number, '.' );
        my $Decimal = ( $IndexDotSeparator >= 0 ) ? ( substr( $Number, $IndexDotSeparator + 1 ) . '0' ) : '00';
        $Decimal = substr( $Decimal, 0, 2 );
        return int($Number) . $Decimal;
    });

    my $aParameterTypes = ['string'];
    $Mockify->addMockWithReturnValueAndParameterCheck('_removeDashesFromGUID', '5502D366B26703F3060AD580920FF5BF', $aParameterTypes);

    return $Mockify->getMockObject();
}

sub _getMockedBasket {
    my $self = shift;

    my ($hasShippingAddress) = @_;

    my $hAttributes = {
        'Alias'             => '1000',
        'LineItemContainer' => $self->_getMockedLineItemContainer(),
        'BillingAddress'    => $self->_getMockedBillingAddress(),
        'Customer'          => $self->_getMockedCustomer(),
        'User'              => $self->_getMockedUser(),
        'ShippingAddress'   => ($hasShippingAddress) ? $self->_getMockedShippingAddress() : undef,
        'ShopAddress'       => $self->_getMockedShopAddress()
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::Basket', $hAttributes );
}

sub _getMockedShippingAddress() {
    my $self = shift;

    my $hAttributes = {
        'Company'     => 'Xavizon',
        'FirstName'   => 'Angela',
        'LastName'    => 'Merckel',
        'EMail'       => 'test@epages.com',
        'Gender'      => 'F',
        'StreetOnly'  => 'Pilatuspool',
        'HouseNumber' => '2',
        'Zipcode'     => '20355',
        'City'        => 'Hamburg',
        'Country'     => { 'Code2' => 'DE', }
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAttributes );
}

sub _getMockedLineItemContainer {
    my $self = shift;

    my $hAttributes = {
        'Payment'        => $self->_getMockedLineItemPaymentDocdataPay(),
        'GrandTotal'     => '197.3',
        'CurrencyID'     => 'GBP',
        'TotalBeforeTax' => '175.67',
        'LineItems'      => $self->_getMockedLineItems(),
        'Shipping'       => $self->_getMockedLineItemShipping()
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes );
}

sub _getMockedLineItems {
    my $self = shift;

    my $hAttributes = {
        'BasePrice'     => '83.30',
        'LineItemPrice' => '83.30',
        'Quantity'      => '1.27',
        'TaxRate'       => '0.19',
        'NameOrAlias'   => 'Leatherman Survival Tool',
        'Product'       => $self->_getMockedProduct( 'Leatherman Survival Tool', 'lt_0401107001' ),
        'instanceOf'    => 'LineItemProduct'
    };
    my $FirstLineItemProduct =
      DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Product::API::Object::LineItemProduct', $hAttributes );

    $hAttributes = {
        'BasePrice'     => '53.50',
        'LineItemPrice' => '107.00',
        'Quantity'      => '2',
        'TaxRate'       => '0.07',
        'NameOrAlias'   => 'Maglite',
        'Product'       => $self->_getMockedProduct( 'Maglite Survival Tool', 'lt_0401107002' ),
        'instanceOf'    => 'LineItemProduct'
    };
    my $SecondLineItemProduct =
      DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Product::API::Object::LineItemProduct', $hAttributes );

    return [ $FirstLineItemProduct, $SecondLineItemProduct ];
}

sub _getMockedProduct {
    my $self = shift;
    my ( $Description, $Alias ) = @_;

    my $hAttributes = {
        'DisplayDescription' => $Description,
        'Alias'              => $Alias,
        'SuperProduct'       => undef
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Product::API::Object::Product', $hAttributes );
}

sub _getMockedLineItemPaymentDocdataPay {
    my $self = shift;

    my $hAttributes = {
        'PaymentMethod' => $self->_getMockedPaymentMethodDocdataPay(),
        'APIVersion'    => '1.2',
        'GUID'          => '5502D366-B267-03F3-060A-D580920FF5BF'
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::DocdataPay::API::Object::LineItemPaymentDocdataPay',
        $hAttributes );
}

sub _getMockedPaymentMethodDocdataPay {
    my $self = shift;

    my $hAttributes = {
        'MerchantName'     => 'merchantname',
        'MerchantPassword' => 'merchantpassword',
        'ProfileName'      => 'profilename'
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::DocdataPay::API::Object::PaymentMethodDocdataPay',
        $hAttributes );
}

sub _getMockedBillingAddress {
    my $self = shift;

    my $hAttributes = {
        'FirstName'   => 'Angela',
        'LastName'    => 'Merckel',
        'EMail'       => 'test@epages.com',
        'Gender'      => 'F',
        'StreetOnly'  => 'Pilatuspool',
        'HouseNumber' => '2',
        'Zipcode'     => '20355',
        'City'        => 'Hamburg',
        'Country'     => { 'Code2' => 'DE', }
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAttributes );
}

sub _getMockedCustomer {
    my $self = shift;

    my $hAttributes = { 'Alias' => '1034' };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Customer::API::Object::Customer', $hAttributes );
}

sub _getMockedUser {
    my $self = shift;

    my $hAttributes = {
        'LanguageID' => '2',
        'ID'         => '1034',
        'instanceOf' => 'User'
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Permission::API::Object::User', $hAttributes );
}

sub _getMockedLineItemShipping {
    my $self = shift;

    my $hAttributes = {
        'LineItemPrice' => '7',
        'TaxRate'       => '0.19',
        'NameOrAlias'   => 'Standard delivery',
        'GUID'          => '54D381FE-DD6D-BE14-D392-7F0001010F2A',
        'instanceOf'    => 'LineItemShipping'
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemShipping',
        $hAttributes );
}

sub _getMockedShopAddress {
    my $self = shift;

    my $hAttributes = { 'VATID' => 'DE 123456' };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAttributes );
}

sub _build_create_request_message {
    my $self = shift;

    my ($hasShippingAddress) = @_;

    my $ExpectedMessage = {
        'version'  => '1.2',
        'merchant' => {
            'name'     => 'merchantname',
            'password' => 'merchantpassword'
        },
        'merchantOrderReference' => '5502D366B26703F3060AD580920FF5BF',
        'paymentPreferences'     => {
            'profile'           => 'profilename',
            'numberOfDaysToPay' => '14'
        },
        'shopper' => {
            'id'   => '1034',
            'name' => {
                'first' => 'Angela',
                'last'  => 'Merckel'
            },
            'email'    => 'test@epages.com',
            'language' => { 'code' => 'en' },
            'gender'   => 'F'
        },
        'totalGrossAmount' => {
            'currency' => 'GBP',
            '_'        => '19730'
        },
        'billTo' => {
            'name' => {
                'first' => 'Angela',
                'last'  => 'Merckel'
            },
            'address' => {
                'street'      => 'Pilatuspool',
                'houseNumber' => '2',
                'postalCode'  => '20355',
                'city'        => 'Hamburg',
                'country'     => { 'code' => 'DE' }
            }
        },
        'invoice' => {
            'totalNetAmount' => {
                'currency' => 'GBP',
                '_'        => '17567'
            },
            'totalVatAmount' => [
                {
                    'currency' => 'GBP',
                    'rate'     => '19',
                    '_'        => '111'
                },
                {
                    'currency' => 'GBP',
                    'rate'     => '19',
                    '_'        => '1330'
                },
                {
                    'currency' => 'GBP',
                    'rate'     => '7',
                    '_'        => '700'
                }
            ],
            'item' => [
                {
                    'number'   => '1',
                    'name'     => 'Standard delivery',
                    'code'     => '54D381FE-DD6D-BE14-D392-7F0001010F2A',
                    'quantity' => {
                        'unitOfMeasure' => 'PCS',
                        '_'             => '1'
                    },
                    'description' => 'Standard delivery',
                    'netAmount'   => {
                        'currency' => 'GBP',
                        '_'        => '588'
                    },
                    'grossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '700'
                    },
                    'vat' => {
                        'rate'   => '19',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '111'
                        },
                    },
                    'totalNetAmount' => {
                        'currency' => 'GBP',
                        '_'        => '588'
                    },
                    'totalGrossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '700'
                    },
                    'totalVat' => {
                        'rate'   => '19',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '111'
                        },
                    }
                },
                {
                    'number'   => '2',
                    'name'     => 'Leatherman Survival Tool',
                    'code'     => 'lt_0401107001',
                    'quantity' => {
                        'unitOfMeasure' => 'PCS',
                        '_'             => '1'
                    },
                    'description' => 'Leatherman Survival Tool',
                    'netAmount'   => {
                        'currency' => 'GBP',
                        '_'        => '7000'
                    },
                    'grossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '8330'
                    },
                    'vat' => {
                        'rate'   => '19',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '1330'
                        },
                    },
                    'totalNetAmount' => {
                        'currency' => 'GBP',
                        '_'        => '7000'
                    },
                    'totalGrossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '8330'
                    },
                    'totalVat' => {
                        'rate'   => '19',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '1330'
                        },
                    }
                },
                {
                    'number'   => '3',
                    'name'     => 'Maglite',
                    'code'     => 'lt_0401107002',
                    'quantity' => {
                        'unitOfMeasure' => 'PCS',
                        '_'             => '2'
                    },
                    'description' => 'Maglite Survival Tool',
                    'netAmount'   => {
                        'currency' => 'GBP',
                        '_'        => '5000'
                    },
                    'grossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '5350'
                    },
                    'vat' => {
                        'rate'   => '7',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '350'
                        },
                    },
                    'totalNetAmount' => {
                        'currency' => 'GBP',
                        '_'        => '10000'
                    },
                    'totalGrossAmount' => {
                        'currency' => 'GBP',
                        '_'        => '10700'
                    },
                    'totalVat' => {
                        'rate'   => '7',
                        'amount' => {
                            'currency' => 'GBP',
                            '_'        => '700'
                        }
                    },
                }
            ],
            'shipTo' => {
                'name' => {
                    'first' => 'Angela',
                    'last'  => 'Merckel'
                },
                'address' => {
                    'street'      => 'Pilatuspool',
                    'houseNumber' => '2',
                    'postalCode'  => '20355',
                    'city'        => 'Hamburg',
                    'country'     => { 'code' => 'DE' }
                }
            },
        }
    };

    if ($hasShippingAddress) {
        $ExpectedMessage->{'invoice'}->{'shipTo'}->{'address'}->{'vatNumber'} = 'DE 123456';
        $ExpectedMessage->{'invoice'}->{'shipTo'}->{'address'}->{'company'}   = 'Xavizon';
    }

    return $ExpectedMessage;
}

__PACKAGE__->runScript();
